=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2006-06-17 13:17:06 +0000
+++ bzrlib/transport/http/__init__.py	2006-06-17 15:52:51 +0000
@@ -193,58 +193,43 @@
         :param offsets: A list of (offset, size) tuples.
         :param return: A list or generator of (offset, data) tuples
         """
-        # Ideally we would pass one big request asking for all the ranges in
-        # one go; however then the server will give a multipart mime response
-        # back, and we can't parse them yet.  So instead we just get one range
-        # per region, and try to coallesce the regions as much as possible.
-        #
-        # The read-coallescing code is not quite regular enough to have a
-        # single driver routine and
-        # helper method in Transport.
-        def do_combined_read(combined_offsets):
-            # read one coalesced block
-            total_size = 0
-            for offset, size in combined_offsets:
-                total_size += size
-            mutter('readv coalesced %d reads.', len(combined_offsets))
-            offset = combined_offsets[0][0]
-            byte_range = (offset, offset + total_size - 1)
-            code, result_file = self._get(relpath, [byte_range])
-            if code == 206:
-                for off, size in combined_offsets:
-                    result_bytes = result_file.read(size)
-                    assert len(result_bytes) == size
-                    yield off, result_bytes
-            elif code == 200:
-                data = result_file.read(offset + total_size)[offset:offset + total_size]
-                pos = 0
-                for offset, size in combined_offsets:
-                    yield offset, data[pos:pos + size]
-                    pos += size
-                del data
-        if not len(offsets):
-            return
-        pending_offsets = deque(offsets)
-        combined_offsets = []
-        while len(pending_offsets):
-            offset, size = pending_offsets.popleft()
-            if not combined_offsets:
-                combined_offsets = [[offset, size]]
-            else:
-                if (len (combined_offsets) < 500 and
-                    combined_offsets[-1][0] + combined_offsets[-1][1] == offset):
-                    # combatible offset:
-                    combined_offsets.append([offset, size])
-                else:
-                    # incompatible, or over the threshold issue a read and yield
-                    pending_offsets.appendleft((offset, size))
-                    for result in do_combined_read(combined_offsets):
-                        yield result
-                    combined_offsets = []
-        # whatever is left is a single coalesced request
-        if len(combined_offsets):
-            for result in do_combined_read(combined_offsets):
-                yield result
+        mutter('readv of %s [%s]', relpath, offsets)
+        ranges = self._offsets_to_ranges(offsets)
+        code, f = self._get(relpath, ranges)
+        for start, size in offsets:
+            f.seek(start, 0)
+            data = f.read(size)
+            assert len(data) == size
+            yield start, data
+
+    def _is_multipart(self, content_type):
+        return content_type.startswith('multipart/byteranges;')
+
+    def _handle_response(self, path, response):
+        """Interpret the code & headers and return a HTTP response.
+
+        This is a factory method which returns an appropriate HTTP response
+        based on the code & headers it's given.
+        """
+        content_type = response.headers['Content-Type']
+        mutter('handling response code %s ctype %s', response.code,
+            content_type)
+
+        if response.code == 206 and self._is_multipart(content_type):
+            # Full fledged multipart response
+            return HttpMultipartRangeResponse(path, content_type, response)
+        elif response.code == 206:
+            # A response to a range request, but not multipart
+            content_range = response.headers['Content-Range']
+            return HttpRangeResponse(path, content_range, response)
+        elif response.code == 200:
+            # A regular non-range response, unfortunately the result from
+            # urllib doesn't support seek, so we wrap it in a StringIO
+            return StringIO(response.read())
+        elif response.code == 404:
+            raise NoSuchFile(path)
+
+        raise BzrError("HTTP couldn't handle code %s", response.code)
 
     def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
@@ -352,8 +337,7 @@
         """
         # We need a copy of the offsets, as the caller might expect it to
         # remain unsorted. This doesn't seem expensive for memory at least.
-        offsets = offsets[:]
-        offsets.sort(key=lambda i: i[0])
+        offsets = sorted(offsets)
 
         start, size = offsets[0]
         prev_end = start + size - 1

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-06-17 15:20:19 +0000
+++ bzrlib/transport/http/_pycurl.py	2006-06-17 15:52:51 +0000
@@ -60,6 +60,66 @@
 register_urlparse_netloc_protocol('http+pycurl')
 
 
+class CurlResponse(object):
+    """This just exists to make the curl response look like a urllib response."""
+
+    def __init__(self, curl_handle):
+        self._data = StringIO()
+        self._header = StringIO()
+        self._curl_handle = curl_handle
+        self._curl_handle.setopt(pycurl.WRITEFUNCTION, self._data.write)
+        self._curl_handle.setopt(pycurl.HEADERFUNCTION, self._header.write)
+        
+        # Will be filled out by calling update()
+        self.headers = None
+        self.code = None
+
+        # Overload some functions used elsewhere
+        self.read = self._data.read
+        self.seek = self._data.seek
+
+    def update(self):
+        """Update self after the action has been performed."""
+        self.code = self._curl_handle.getinfo(pycurl.HTTP_CODE)
+        self._build_headers()
+        self._data.seek(0, 0)
+
+    def _build_headers(self):
+        """Parse the headers into RFC822 format"""
+
+        # TODO: jam 20060617 We probably don't need all of these
+        #       headers, but this is what the 'curl' wrapper around
+        #       pycurl does
+        self._header.seek(0, 0)
+        url = self._curl_handle.getinfo(pycurl.EFFECTIVE_URL)
+        if url[:5] in ('http:', 'https:'):
+            self._header.readline()
+            m = mimetools.Message(self._header)
+        else:
+            m = mimetools.Message(StringIO())
+        m['effective-url'] = url
+        m['http-code'] = str(self._curl_handle.getinfo(pycurl.HTTP_CODE))
+        # jam 20060617 These aren't used, so don't bother parsing them
+        # m['total-time'] = str(self._curl_handle.getinfo(pycurl.TOTAL_TIME))
+        # m['namelookup-time'] = str(self._curl_handle.getinfo(pycurl.NAMELOOKUP_TIME))
+        # m['connect-time'] = str(self._curl_handle.getinfo(pycurl.CONNECT_TIME))
+        # m['pretransfer-time'] = str(self._curl_handle.getinfo(pycurl.PRETRANSFER_TIME))
+        # m['redirect-time'] = str(self._curl_handle.getinfo(pycurl.REDIRECT_TIME))
+        # m['redirect-count'] = str(self._curl_handle.getinfo(pycurl.REDIRECT_COUNT))
+        # m['size-upload'] = str(self._curl_handle.getinfo(pycurl.SIZE_UPLOAD))
+        # m['size-download'] = str(self._curl_handle.getinfo(pycurl.SIZE_DOWNLOAD))
+        # m['speed-upload'] = str(self._curl_handle.getinfo(pycurl.SPEED_UPLOAD))
+        # m['header-size'] = str(self._curl_handle.getinfo(pycurl.HEADER_SIZE))
+        # m['request-size'] = str(self._curl_handle.getinfo(pycurl.REQUEST_SIZE))
+        # m['content-length-download'] = str(self._curl_handle.getinfo(pycurl.CONTENT_LENGTH_DOWNLOAD))
+        # m['content-length-upload'] = str(self._curl_handle.getinfo(pycurl.CONTENT_LENGTH_UPLOAD))
+
+        # Shouldn't this be 'Content-Type', I'm hoping that was already
+        # parsed by the mimetools section.
+        # m['content-type'] = (self._curl_handle.getinfo(pycurl.CONTENT_TYPE) or '').strip(';')
+        self.headers = m
+
+
 class PyCurlTransport(HttpTransportBase):
     """http client transport using pycurl
 
@@ -112,66 +172,19 @@
             curl = self._range_curl
 
         abspath = self._real_abspath(relpath)
-        sio = StringIO()
-        header_sio = StringIO()
         curl.setopt(pycurl.URL, abspath)
         self._set_curl_options(curl)
-        curl.setopt(pycurl.WRITEFUNCTION, sio.write)
         curl.setopt(pycurl.NOBODY, 0)
-        curl.setopt(pycurl.HEADERFUNCTION, header_sio.write)
+
+        response = CurlResponse(curl)
 
         if ranges is not None:
-            assert len(ranges) == 1
-            # multiple ranges not supported yet because we can't decode the
-            # response
-            curl.setopt(pycurl.RANGE, '%d-%d' % ranges[0])
+            curl.setopt(pycurl.RANGE, self._range_header(ranges))
         self._curl_perform(curl)
-        info = self._get_header_info(header_sio, curl)
-        code = info['http-code']
-        if code == '404':
-            raise NoSuchFile(abspath)
-        elif code == '200':
-            sio.seek(0)
-            return code, sio
-        elif code == '206' and (ranges is not None):
-            sio.seek(0)
-            return code, sio
-        elif code == 0:
+        response.update()
+        if response.code == 0:
             self._raise_curl_connection_error(curl)
-        else:
-            self._raise_curl_http_error(curl)
-
-    def _get_header_info(self, header, curl_handle):
-        """Parse the headers into RFC822 format"""
-
-        # TODO: jam 20060617 We probably don't need all of these
-        #       headers, but this is what the 'curl' wrapper around
-        #       pycurl does
-        header.seek(0, 0)
-        url = curl_handle.getinfo(pycurl.EFFECTIVE_URL)
-        if url[:5] in ('http:', 'https:'):
-            header.readline()
-            m = mimetools.Message(header)
-        else:
-            m = mimetools.Message(StringIO())
-        m['effective-url'] = url
-        m['http-code'] = str(curl_handle.getinfo(pycurl.HTTP_CODE))
-        m['total-time'] = str(curl_handle.getinfo(pycurl.TOTAL_TIME))
-        m['namelookup-time'] = str(curl_handle.getinfo(pycurl.NAMELOOKUP_TIME))
-        m['connect-time'] = str(curl_handle.getinfo(pycurl.CONNECT_TIME))
-        m['pretransfer-time'] = str(curl_handle.getinfo(pycurl.PRETRANSFER_TIME))
-        m['redirect-time'] = str(curl_handle.getinfo(pycurl.REDIRECT_TIME))
-        m['redirect-count'] = str(curl_handle.getinfo(pycurl.REDIRECT_COUNT))
-        m['size-upload'] = str(curl_handle.getinfo(pycurl.SIZE_UPLOAD))
-        m['size-download'] = str(curl_handle.getinfo(pycurl.SIZE_DOWNLOAD))
-        m['speed-upload'] = str(curl_handle.getinfo(pycurl.SPEED_UPLOAD))
-        m['header-size'] = str(curl_handle.getinfo(pycurl.HEADER_SIZE))
-        m['request-size'] = str(curl_handle.getinfo(pycurl.REQUEST_SIZE))
-        m['content-length-download'] = str(curl_handle.getinfo(pycurl.CONTENT_LENGTH_DOWNLOAD))
-        m['content-length-upload'] = str(curl_handle.getinfo(pycurl.CONTENT_LENGTH_UPLOAD))
-        m['content-type'] = (curl_handle.getinfo(pycurl.CONTENT_TYPE) or '').strip(';')
-        return m
-
+        return response.code, self._handle_response(relpath, response)
 
     def _raise_curl_connection_error(self, curl):
         curl_errno = curl.getinfo(pycurl.OS_ERRNO)

=== modified file 'bzrlib/transport/http/_response.py'
--- bzrlib/transport/http/_response.py	2006-06-16 08:38:21 +0000
+++ bzrlib/transport/http/_response.py	2006-06-17 15:52:51 +0000
@@ -28,8 +28,6 @@
 from bzrlib.errors import TransportError
 from bzrlib.trace import mutter
 
-__all__ = ['handle_response']
-
 
 class ResponseRange(object):
     def __init__(self, ent_start, ent_end, data_start):
@@ -111,6 +109,7 @@
         if self._pos < 0:
             self._pos = 0
 
+
 class HttpRangeResponse(RangeFile):
     """A single-range HTTP response."""
 

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2006-06-16 08:38:21 +0000
+++ bzrlib/transport/http/_urllib.py	2006-06-17 15:52:51 +0000
@@ -52,50 +52,6 @@
         """Set the base path where files will be stored."""
         super(HttpTransport_urllib, self).__init__(base)
 
-    def readv(self, relpath, offsets):
-        """Get parts of the file at the given relative path.
-
-        :param offsets: A list of (offset, size) tuples.
-        :param return: A list or generator of (offset, data) tuples
-        """
-        mutter('readv of %s [%s]', relpath, offsets)
-        ranges = self._offsets_to_ranges(offsets)
-        code, f = self._get(relpath, ranges)
-        for start, size in offsets:
-            f.seek(start, 0)
-            data = f.read(size)
-            assert len(data) == size
-            yield start, data
-
-    def _is_multipart(self, content_type):
-        return content_type.startswith('multipart/byteranges;')
-
-    def _handle_response(self, path, response):
-        """Interpret the code & headers and return a HTTP response.
-
-        This is a factory method which returns an appropriate HTTP response
-        based on the code & headers it's given.
-        """
-        content_type = response.headers['Content-Type']
-        mutter('handling response code %s ctype %s', response.code,
-            content_type)
-
-        if response.code == 206 and self._is_multipart(content_type):
-            # Full fledged multipart response
-            return HttpMultipartRangeResponse(path, content_type, response)
-        elif response.code == 206:
-            # A response to a range request, but not multipart
-            content_range = response.headers['Content-Range']
-            return HttpRangeResponse(path, content_range, response)
-        elif response.code == 200:
-            # A regular non-range response, unfortunately the result from
-            # urllib doesn't support seek, so we wrap it in a StringIO
-            return StringIO(response.read())
-        elif response.code == 404:
-            raise NoSuchFile(path)
-
-        raise BzrError("HTTP couldn't handle code %s", response.code)
-
     def _get(self, relpath, ranges):
         path = relpath
         try:

